/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef CAPPEDRECTANGLE_H
#define CAPPEDRECTANGLE_H

#include "PhysicalObject.h"
class Border;
class Circle;

/** Defines a rectangle having the thin ends capped with half circles. 
The base position (relative to which the rotation angle is measured) is along the x axis, 
like this: (______). l is the half length of the rectangle, R its half width. 
The end half circles have a radius R. */

class CappedRectangle : public PhysicalObject {

public:
	CappedRectangle(real l, real R,                    
		real x = 0, real y = 0, real alpha = 0, std::string label="",
		int nSensorsLateral=0, int nSensorsSemicircle=0,
		real saturationForce=0.5,
		Color outlineColor = GUI::colorBlack, Color fillColor = GUI::colorTransparent
	);
	/** Destructor, does nothing. */
	virtual ~CappedRectangle();

	/** Sets the mass based on the dimensions and the density */
	virtual void computeMass(real density = ThyrixParameters::defaultDensity);

	/** Computes the moment of inertia around the z axis that passes through the center */
	virtual void computeInertia();

  virtual bool detectContacts(PhysicalObject* object,  GlobalContactInfoVector* contacts) {
      return object->detectContacts(this, contacts);
  }

	/** Tests for contacts between this object and a border. */
	bool detectContacts(Border* border, GlobalContactInfoVector* contacts);

	/** Tests for contacts between this object and a circle. */
	bool detectContacts(Circle* circle,  GlobalContactInfoVector* contacts);

	/** Tests for contacts between this object and another capped rectangle */
	bool detectContacts(CappedRectangle* capsule, GlobalContactInfoVector* contacts);

	/** Detects whether a click of the mouse at rMouse has touched the object. In case of 
	contact, returns true and sets p to the relative vector between the center of the object
	and the contact point; p is expressed in the object reference frame because it rotates with the object
	while the object is dragged.  */
	bool detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object);

	virtual void computeBox(){
		cosAlpha=cos(alpha);
		sinAlpha=sin(alpha);
		Vector2 rt(fabs(l*cosAlpha),fabs(l*sinAlpha));
		rt+=R+ThyrixParameters::halfEpsilonContact;
		boxMin=r; boxMin-=rt;
		boxMax=r; boxMax+=rt;
	}

	virtual void setSensor(ContactInfo* contact);
	
	/** Draws the object on the user interface. */
	void draw(GUI *gui);

	/** Draws the object's tactile sensors. */
	void drawSensors(GUI *gui);

//private:
	/** Dimensions; l is the half length of the rectangle, R its half width. 
	The end half circles will have radius R.*/
	real l;
	real R;

	/** Cosinus of alpha. Always updated. */
	real cosAlpha;
	/** Sinus of alpha. Always updated. */
	real sinAlpha;

	/** The number of tactile sensors per lateral of the rectangle. */
	unsigned nSensorsLateral;

	/** The number of tactile sensors per end semicircle. */
	unsigned nSensorsSemicircle;

private:
	/** Gets the index of a tactile sensor on a semicircle, given the relative vector p
	from the end of the capsule's axis to the contact point, and the sign (+ or -) that 
	determins one of the 2 ends of the capsule's axis. */
	int getISensorSemicircle(Vector2& p, int sign);

	/** Gets the index of a tactile sensor on a lateral of the capsule,
	given the side of the capsule determined by the normal component (relative to the capsule's axis)
	of the vector p, and the linear coordinates of the ends of the lateral contact on the capsule's
	axis. sinAlpha and cosAlpha are passed as they are already computed. */
	void setISensorLateral(int& i1, int& i2, Vector2& p, real e1, real e2);

	/** Returns the index of the lateral sensor corresponding to a contact given by e and the vector p 
	relative to the center of the capsule. The projection of the contact point on the capsule axis is at a
	distance e from the capsule center (e can be positive or negative). */
	int getISensorLateral(Vector2& p, real e);

	/** Returns the index of the lateral sensor corresponding to a contact given by e and the side
	of the capsule (a positive value corresponds to the top side of the capsule, a negative value to 
	the bottom side). The projection of the contact point on the capsule axis is at a
	distance e from the capsule center (e can be positive or negative). */
	int getISensorLateral(real side, real e);

	/** Returns the side on which exists a contact given by the vector p relative to the center of the capsule.
	A positive returned value corresponds to the top side of the capsule, a negative value to the bottom side.*/
	real getLateralSide(Vector2& p){
		return -p.x*sinAlpha+p.y*cosAlpha;
	}

};

#endif //CAPPEDRECTANGLE_H
