/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Circle.h: interface for the Circle class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CIRCLE_H
#define CIRCLE_H

#include "PhysicalObject.h"
#include "Border.h"

class Simulator;

/** A circle. It has radius R. */

class Circle : public PhysicalObject {

public:
	/** Constructor, sets the position and radius, the tactile sensors (if any),
	computes the mass and inertia based on default density. 
  */
	Circle(real R, real x = 0, real y = 0, std::string label="", 
		int nSensors=0, real sensorsStartAngle=0, real saturationForce=0.5,
		Color outlineColor = GUI::colorBlack, Color fillColor = GUI::colorTransparent);

	/** Destructor, does nothing. */
	virtual ~Circle();
	
	/** Radius of the circle */
	real R;

	/** Sets the radius of the circle, computes the mass and inertia based on default density. */
	void setRadius (real R);
	
	/** Sets the mass based on the radius and the density. */
	virtual void computeMass(real density = ThyrixParameters::defaultDensity);
	
	/** Computes the moment of inertia around the z axis that passes through the center, based on 
	the mass and the radius. */
	virtual void computeInertia();

	/** Tells that this object has circular symmetry. */
	virtual int isCircle()									{ return 1; }
	
	/** Redirects contact detection.*/
  virtual bool detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts) { 
      return object->detectContacts(this, contacts);
  }

	/** Tests for contacts between this object and a border. */
	virtual bool detectContacts(Border* border, GlobalContactInfoVector* contacts);

	/** Tests for contacts between this object and a circle. */
	virtual bool detectContacts(Circle* circle,  GlobalContactInfoVector* contacts);

	/** Detects whether a click of the mouse at rMouse has touched the object. In case of 
	contact, returns true and sets p to the relative vector between the center of the object
	and the contact point; p is expressed in the object reference frame because it rotates with the object
	while the object is dragged. */
	virtual bool detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object);

	/** Computes the bounding box. */
	virtual void computeBox(){;
		real RE=R+ThyrixParameters::halfEpsilonContact;
		boxMin=r; boxMin-=RE;
		boxMax=r; boxMax+=RE;
	}

	/** The start angle for the contact sensors; 0..2Pi */
	real sensorsStartAngle;

	/** Returns the index of the tactile sensor activated by a contact
	at relative position p. */
	int getISensor(Vector2& p);

	virtual void setSensor(ContactInfo* contact);

	/** Draws the circle on the user interface. */
	virtual void draw(GUI *gui);

	/** Draws the activation of the circle's contact sensors. */
	void drawSensors(GUI *gui);
};

#endif //CIRCLE_H
