/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#include "ComposedPhysicalObject.h"
#include "Simulator.h"
#include "purgeContainer.h"
#include <string>

ComposedPhysicalObject::ComposedPhysicalObject(std::string label) :
	PhysicalObject(label){
}

ComposedPhysicalObject::ComposedPhysicalObject(real x, real y, real alpha, std::string label) :
	PhysicalObject(label){
	r.setXY(x,y);
	this->alpha=alpha;
}

ComposedPhysicalObject::~ComposedPhysicalObject(){
	deleteObjects();
}



void ComposedPhysicalObject::addObject(PhysicalObject* object, real x, real y, real alpha){
	object->parent=this;
	object->relativeR.setXY(x,y);
	object->relativeAlpha=alpha;
	object->label=label+": "+object->label;
	objects.push_back(object);
	computeMassProperties();
}

void ComposedPhysicalObject::deleteObjects(){
	purgeContainer(objects);	
}


void ComposedPhysicalObject::computeMassProperties(){
	Vector2 v;
	PhysicalObject* object;
	m=0;
	I=0;
	s.setToZero();
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
										end=objects.end(); it!=end; ++it){
		object=*it;
		assert(object);
		m+=object->m;		//add member object mass to total mass
		v=object->relativeR;//position of the member object relative to the composed object
		I+=object->m*v.getSquaredModule()+object->I;
		v*=object->m;		
		s+=v;
	}
	s/=m;
}

void ComposedPhysicalObject::computeMemberPositions(){
	Vector2 v;
	PhysicalObject* object;
	boxMin.setXY(ThyrixParameters::infinity,ThyrixParameters::infinity);
	boxMax.setXY(-ThyrixParameters::infinity,-ThyrixParameters::infinity);
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
										end=objects.end(); it!=end; ++it){
		object=*it;
		v=object->relativeR;	//the position relative to the ORS
		v.rotate(alpha);
		v+=r;					//add the position of ORS relative to LRS
		object->rOld=object->r;
		object->r=v;		//update the position of member object relative to LRS
		object->alphaOld=object->alpha;
		object->alpha=alpha+object->relativeAlpha;
		object->computeBox();
		boxMin.updateMin(object->boxMin);
		boxMax.updateMax(object->boxMax);
	}
}

void ComposedPhysicalObject::registerPrimitives(Simulator* simulator){
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
            end=objects.end(); it!=end; ++it) {
		simulator->registerPrimitive(*it);
  }
}

bool ComposedPhysicalObject::detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts) {
	bool isContact=false;
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
																		end=objects.end(); it!=end; ++it)
		isContact = (*it)->detectContacts(object, contacts) || isContact;
	return isContact;
}

bool ComposedPhysicalObject::detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object){
	bool isContact=false;
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
			end=objects.end(); it!=end && !isContact; ++it){
			isContact = (*it)->detectMouseContact(rMouse, p, object);
			if (isContact){
				p.rotate(-relativeAlpha);
				p+=(*it)->relativeR; //p is now relative to the origin of the composed object
				object=this;
			}
		}
	return isContact;
}

void ComposedPhysicalObject::setSensors(){
	ContactInfo* current;
	for(ContactInfoPVector::iterator it=contacts.begin(), end=contacts.end();
        it!=end;++it){
		current=*it;
		if(current->iSensor>=0){
			current->object->setSensor(current);
		}
	}
}

void ComposedPhysicalObject::resetSensors(){
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
            end=objects.end(); it!=end; ++it) {
		(*it)->resetSensors();
  }
}

void ComposedPhysicalObject::draw(GUI *gui){
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
            end=objects.end(); it!=end; ++it) {
		(*it)->draw(gui);
	}
	//drawContactForces(gui);
}

void ComposedPhysicalObject::setOutlineColor(Color color){
	outlineColor=color;
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
            end=objects.end(); it!=end; ++it) {
		(*it)->setOutlineColor(color);
	}
}

void ComposedPhysicalObject::setFillColor(Color color){
	fillColor=color;
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
            end=objects.end(); it!=end; ++it) {
		(*it)->setFillColor(color);
	}
}