/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef VECTOR2_H
#define VECTOR2_H

#include "ThyrixParameters.h"


#include <assert.h>

/** A 2D vector. */

class Vector2 {

public:
	
	/** The default constructor, sets the vector to 0. */
	Vector2 ()									{ x=0.0;	y=0.0; }
	
	/** Constructor. */
	Vector2 (real x, real y)				{ this->x=x; this->y=y;	}
	
	/** Destructor; does nothing. */
	virtual ~Vector2 ()						{}

	real x;
	real y;

	/** Sets the vector to 0. */
	void setToZero()							{ x=0.0;	y=0.0; }

	/** Sets x and y components. */
	void setXY(real x, real y)				{ this->x=x; this->y=y;	}

	/** Sets element i to value value; i is numbered starting from 1 (i=1,2). */
	void setElement(int i, real value)	{assert(i>=1 && i<=2);	i==1?	x=value :	y=value; }

	/** Gets the module of the vector: sqrt(x^2+y^2). */
	real getModule()							{ 
		double ax=fabs(x), ay=fabs(y);
		if(ax>ay) return ax*sqrt(1.0+sqr(ay/ax));
		else return (ay==0.0 ? 0.0 : ay*sqrt(1.0+sqr(ax/ay)));
	}

	/** Gets the squared module of the vector: x^2+y^2. */
	real getSquaredModule()					{ return x*x+y*y; }

	/** Normalizes the vector to have unit module */
	void normalize() { 
		real module=getModule(); 
		assert(module!=0);
		x/=module; y/=module;
	}

	/** Set vector components to the minimum of either current components or r components. */
	void updateMin(const Vector2& r){
		if(r.x<x) x=r.x;
		if(r.y<y) y=r.y;
	}

	/** Set vector components to the maximum of either current components or r components. */
	void updateMax(const Vector2& r){
		if(r.x>x) x=r.x;
		if(r.y>y) y=r.y;
	}

	
	/** Rotates the vector with angle alpha. If the vector is initially expressed in a reference
	system rotated with alpha relative to the laboratory reference system, after rotation the vector
	will be expressed in the laboratory reference system. */
	void rotate(real alpha) {
		real xOld=x;
		real c=cos(alpha);
		real s=sin(alpha);
		x=x*c-y*s;
		y=xOld*s+y*c;
	}

	/** The vector r becomes w x r (cross product), w being a vector having just a z axis component. */
	void cross(real w){
		real xOld=x;
		x=-w*y;
		y=w*xOld;
	}

	
	/** Gets element i; i is numbered starting from 0 (i=0,1). */
	real operator[](int i){
		assert(i>=0 && i<=1);
		if (i==0)
			return x;
		else return y;
	}

	Vector2 operator+(const Vector2 &v) const {
		return Vector2(x + v.x, y + v.y);
	}

	Vector2 operator-(const Vector2 &v) const {
			return Vector2(x - v.x, y - v.y);
	}

	Vector2 operator*(const real r) const {
			return Vector2(x * r, y * r);
	}

	Vector2& operator+=(const Vector2& v)	{ x+=v.x; y+=v.y; return *this; }
	Vector2& operator-=(const Vector2& v)	{ x-=v.x; y-=v.y; return *this; }
	Vector2& operator+=(const real r)		{ x+=r; y+=r; return *this; }
	Vector2& operator-=(const real r)		{ x-=r; y-=r; return *this; }
	Vector2& operator*=(const real r)		{ x*=r; y*=r; return *this; }
	Vector2& operator/=(const real r)		{ assert(r!=0.0); x/=r; y/=r; return *this; }
	/** The scalar product. */
	real operator*(Vector2 v)					{ return x*v.x+y*v.y; }
	/** The vector product (having a component only on the z axis; returns this component). */
	real operator^(Vector2 v)					{ return x*v.y-y*v.x; }
};


#endif //VECTOR2_H
