/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ArticulatedAgentQuasistatic.cpp: implementation of the ArticulatedAgentQuasistatic class.
//
//
//////////////////////////////////////////////////////////////////////


#include "ArticulatedAgentQuasistatic.h"
#include "Simulator.h"
#include "ContactInfo.h"
#include <algorithm>



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

    
ArticulatedAgentQuasistatic::ArticulatedAgentQuasistatic(std::string label)
	:ArticulatedAgentBase(label){
}

ArticulatedAgentQuasistatic::~ArticulatedAgentQuasistatic(){
}


void ArticulatedAgentQuasistatic::forwardKinematics(){
	Vector2 rt, rc;
	real alphat;
	ArticulatedLink* link;
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd;
	ArticulatedLinkPVector::iterator linksI, linksEnd;
	for(limbsI=limbs.begin(), limbsEnd=limbs.end(); limbsI!=limbsEnd; limbsI++){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; linksI++){
			link=*linksI;
			assert(link);
			link->computeSinCos();
			if(link->parentLink==NULL) {
				rt=r;
				alphat=alpha+limb->theta;
				//update alpha
				link->alpha = alphat + link->theta;
				//update r
				rc.setXY(limb->l,0);
				rc.rotate(alphat);
				rt+=rc;
				link->r=rt;
			} else {
				rt=link->parentLink->r;
				alphat=link->parentLink->alpha;
				//update alpha
				link->alpha = alphat + link->theta;
				//update r
				rc.setXY(link->parentLink->l,0);
				rc.rotate(alphat);
				rt+=rc;
				link->r=rt;
			}				
			link->computeMemberPositions();
			link->computeMotorTorque();
		}
	}
}

void ArticulatedAgentQuasistatic::backwardDynamics(){
	ArticulatedLimb* limb;
	ArticulatedLink* link;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd;
	ArticulatedLinkPVector::reverse_iterator linksI, linksEnd;
	ArticulatedLink* childLink;
	ArticulatedLinkPVector::iterator cli, clend;
	real N11,N12,N22,Na,Nb,Nc,lNa;
	SymmetricMatrix3 N;
	Vector3 betaLocal;
	Vector3 rt;
	Vector3 Kt;
	Vector3 n;
	Vector2 r2;
	ContactInfo* contactObject;
	LinkContactInfoPVector::iterator Ki1, Ki2, Kend1, Kend2;
	ContactInfoPVector::iterator ci, cend;
	int isNewContact;

	//body properties
	IStar=IStar0;
	r2=externalForce;
	r2.rotate(-alpha); //the components of beta are expressed in the object frame of reference
	rt.setXYZ(externalTorque,r2.x,r2.y);
	betaExternal+=rt;
	beta=betaExternal;
	betaStar=beta;
	//reset external force and torque
	externalForce.setToZero();
	externalTorque=0;
	/*
	It is simplified from:
	beta.setToZero(); //there is no coriolis component in the quasistaic case
	beta+=betaExternal; //thus beta=betaExternal
	betaStar=beta;
	Note that for classical mechanics, beta is not 0, we cannot do this simplification:
	beta.setXYZ(0.0,s.x,s.y);
	beta*=m*sqr(omega);
	beta thus varies in time. We will have then 
	beta+=betaExternal;
	betaStar=beta;
	*/
	//delete old body K
	deleteK();

	for(limbsI=limbs.begin(), limbsEnd=limbs.end(); limbsI!=limbsEnd; ++limbsI){
		limb=*limbsI;
		//reverse links traversal
		for (linksI=limb->links.rbegin(), linksEnd=limb->links.rend(); linksI!=linksEnd; ++linksI){
			link=*linksI;
			assert(link);

			r2=link->externalForce;
			r2.rotate(-link->alpha);
			link->betaExternal.setXYZ(link->externalTorque,r2.x,r2.y);
			link->beta=link->betaExternal;
			link->betaStar=link->beta;
			//reset external force and torque
			link->externalForce.setToZero();
			link->externalTorque=0;
			/*
			It is simplified from:
			link->beta.setToZero();
			link->beta+=link->betaExternal;
			link->betaStar=link->beta;
			Note that for classical mechanics, beta is not 0, we cannot do this simplification:
			link->beta.setXYZ(0.0,link->s.x,link->s.y);
			link->beta*=link->m*sqr(link->omega);
			beta thus varies in time. We will have then 
			link->beta+=link->betaExternal;
			link->betaStar=link->beta;
			*/
			//delete old link K
			link->deleteK();

			if(link->childLinks.size()){
				/*it is not a tip link;
				if it is a tip link, IStar already set to IStar0 by constructor, 
				and betaStar already set to beta, above */
				link->IStar=link->IStar0;
				for(cli=link->childLinks.begin(), clend=link->childLinks.end(); cli!=clend; ++cli){
					childLink=*cli;
					assert(childLink);
					//computation of N
					N11=-sqr(childLink->IStar.getElement(1,2))/childLink->IStar.getElement(1,1)+
						childLink->IStar.getElement(2,2);
					N12=-childLink->IStar.getElement(1,2)*childLink->IStar.getElement(1,3)/
						childLink->IStar.getElement(1,1)+
						childLink->IStar.getElement(2,3);
					N22=-sqr(childLink->IStar.getElement(1,3))/childLink->IStar.getElement(1,1)+
						childLink->IStar.getElement(3,3);
					//rotation of N
					Na=N11*childLink->sin2+N22*childLink->cos2+2*N12*childLink->sinCos;
					Nb=(N11-N22)*childLink->sinCos+N12*(childLink->cos2-childLink->sin2);
					Nc=N11*childLink->cos2+N22*childLink->sin2-2*N12*childLink->sinCos;
					lNa=link->l*Na;
					N.setElements(link->l * lNa, link->l * Nb, lNa, Nc, Nb, Na);
					link->IStar+=N;
					//compute betaStar
					betaLocal=childLink->betaStar;
					//the n_i (M_i star)/(m_i star) term
					//n_i
					n.setXYZ(childLink->IStar.getElement(1,1),childLink->IStar.getElement(1,2), 
						childLink->IStar.getElement(1,3)); 
					//rt=n_i * (M_i star)/(m_i star)
					rt=n;
					rt*=(childLink->motorTorque+childLink->betaStar.x)/
						childLink->IStar.getElement(1,1);
					betaLocal-=rt;
					//rotate
					rt.z=betaLocal.y*childLink->sinTheta+betaLocal.z*childLink->cosTheta;
					rt.x=betaLocal.x+link->l*(rt.z);
					rt.y=betaLocal.y*childLink->cosTheta-betaLocal.z*childLink->sinTheta;
					link->betaStar+=rt;
					
					//add the K's
					for(Ki1=childLink->K.begin(), Kend1=childLink->K.end(); 
								Ki1!=Kend1; Ki1++){
						Kt=(*Ki1)->v;
						rt=n;
						rt*=-Kt.x/childLink->IStar.getElement(1,1);
						Kt+=rt;
						//rotate
						rt.z=Kt.y*childLink->sinTheta+Kt.z*childLink->cosTheta;
						rt.x=Kt.x+link->l*(rt.z);
						rt.y=Kt.y*childLink->cosTheta-Kt.z*childLink->sinTheta;
						isNewContact=1;
						for(Ki2=link->K.begin(), Kend2=link->K.end();
								Ki2!=Kend2 && isNewContact; Ki2++)
							if((*Ki1)->alpha==(*Ki2)->alpha){
								isNewContact=0;
								(*Ki2)->v+=rt;
							}
						if(isNewContact){
							link->K.push_back(new LinkContactInfo((*Ki1)->alpha, rt));
						}
					}
					//add the K for torque contact
					if(childLink->torqueContact!=NULL){
						Kt=n;
						Kt*=-childLink->torqueContact->sigma/childLink->IStar.getElement(1,1);
						//rotate
						rt.z=Kt.y*childLink->sinTheta+Kt.z*childLink->cosTheta;
						rt.x=Kt.x+link->l*(rt.z);
						rt.y=Kt.y*childLink->cosTheta-Kt.z*childLink->sinTheta;
						link->K.push_back(new LinkContactInfo(childLink->torqueContact->alpha, rt));
					}
				}	//for child links
			}		//if it has child links
			//add contacts of the current link to K
			for(ci=link->contacts.begin(), cend=link->contacts.end(); ci!=cend; ++ci){
				contactObject=*ci;
				r2=contactObject->n;
				r2.rotate(-link->alpha);
				rt.setXYZ(contactObject->pxn, r2.x, r2.y);
				rt*=contactObject->sigma;
				isNewContact=1;
				for(Ki1=link->K.begin(), Kend1=link->K.end();
								Ki1!=Kend1 && isNewContact; Ki1++)
					if(contactObject->alpha==(*Ki1)->alpha){
						isNewContact=0;
						(*Ki1)->v+=rt;
					}
				if(isNewContact){
					link->K.push_back(new LinkContactInfo(contactObject->alpha, rt));
				}
			}
		}	//for each link
		//**** Body computations ***********************************
		//compute force exercited on the body by the current limb
		childLink=*(limb->links.begin()); //the first link of the limb
		assert(childLink);
		//computation of N
		N11=-sqr(childLink->IStar.getElement(1,2))/childLink->IStar.getElement(1,1)+
			childLink->IStar.getElement(2,2);
		N12=-childLink->IStar.getElement(1,2)*childLink->IStar.getElement(1,3)/
			childLink->IStar.getElement(1,1)+
			childLink->IStar.getElement(2,3);
		N22=-sqr(childLink->IStar.getElement(1,3))/childLink->IStar.getElement(1,1)+
			childLink->IStar.getElement(3,3);
		//rotation of N
		Na=N11*childLink->sin2+N22*childLink->cos2+2*N12*childLink->sinCos;
		Nb=(N11-N22)*childLink->sinCos+N12*(childLink->cos2-childLink->sin2);
		Nc=N11*childLink->cos2+N22*childLink->sin2-2*N12*childLink->sinCos;
		//second rotation of N
		N.setElements(limb->l*limb->l*Na,	limb->l*(Nb*limb->cosTheta-Na*limb->sinTheta),
							limb->l*(Nb*limb->sinTheta+Na*limb->cosTheta),
							Nc*limb->cos2-2*Nb*limb->sinCos+Na*limb->sin2, 
							(Nc-Na)*limb->sinCos+Nb*(limb->cos2-limb->sin2),
							Nc*limb->sin2+2*Nb*limb->sinCos+Na*limb->cos2);
		IStar+=N;
		//compute betaStar
		betaLocal=childLink->betaStar;
		//the n_i (M_i star)/(m_i star) term
		n.setXYZ(childLink->IStar.getElement(1,1),childLink->IStar.getElement(1,2), 
			childLink->IStar.getElement(1,3));
		rt=n;
		rt*=-(childLink->motorTorque+childLink->betaStar.x)/
			childLink->IStar.getElement(1,1);
		betaLocal+=rt;
		//rotate
		rt.z=betaLocal.y*childLink->sinTheta+betaLocal.z*childLink->cosTheta;
		rt.x=betaLocal.x+limb->l*(rt.z);
		rt.y=betaLocal.y*childLink->cosTheta-betaLocal.z*childLink->sinTheta;
		betaLocal=rt;
		//second rotation
		rt.x=betaLocal.x;
		rt.y=betaLocal.y*limb->cosTheta-betaLocal.z*limb->sinTheta;
		rt.z=betaLocal.y*limb->sinTheta+betaLocal.z*limb->cosTheta;
		betaStar+=rt;
		
		//compute the K of the body
		for(Ki1=childLink->K.begin(), Kend1=childLink->K.end(); Ki1!=Kend1; Ki1++){
			Kt=(*Ki1)->v;
			rt=n;
			rt*=-Kt.x/childLink->IStar.getElement(1,1);
			Kt+=rt;
			//rotate
			rt.z=Kt.y*childLink->sinTheta+Kt.z*childLink->cosTheta;
			rt.x=Kt.x+limb->l*(rt.z);
			rt.y=Kt.y*childLink->cosTheta-Kt.z*childLink->sinTheta;
			Kt=rt;
			//second rotation
			rt.x=Kt.x;
			rt.y=Kt.y*limb->cosTheta-Kt.z*limb->sinTheta;
			rt.z=Kt.y*limb->sinTheta+Kt.z*limb->cosTheta;
			isNewContact=1;
			for(Ki2=K.begin(), Kend2=K.end(); 
			 Ki2!=Kend2 && isNewContact; Ki2++)
				if((*Ki1)->alpha==(*Ki2)->alpha){
					isNewContact=0;
					(*Ki2)->v+=rt;
				}
			if(isNewContact){
				K.push_back(new LinkContactInfo((*Ki1)->alpha, rt));
			}
		}
		//add the K for torque contact
		if(childLink->torqueContact!=NULL){
			Kt=n;
			Kt*=-childLink->torqueContact->sigma/childLink->IStar.getElement(1,1);
			//rotate
			rt.z=Kt.y*childLink->sinTheta+Kt.z*childLink->cosTheta;
			rt.x=Kt.x+limb->l*(rt.z);
			rt.y=Kt.y*childLink->cosTheta-Kt.z*childLink->sinTheta;
			Kt=rt;
			//second rotation
			rt.x=Kt.x;
			rt.y=Kt.y*limb->cosTheta-Kt.z*limb->sinTheta;
			rt.z=Kt.y*limb->sinTheta+Kt.z*limb->cosTheta;
			K.push_back(new LinkContactInfo(childLink->torqueContact->alpha, rt));
		}
	} // for each limb

	real determinant=IStar.getDeterminant();
	IStar.mirror();
	computeBodyDerivativesWithoutContacts(determinant);

	//add body contacts to K
	for(ci=contacts.begin(), cend=contacts.end(); ci!=cend; ci++){
		contactObject=*ci;
		r2=contactObject->n;
		r2.rotate(-alpha);
		rt.setXYZ(contactObject->pxn, r2.x, r2.y);
		rt*=contactObject->sigma;
		isNewContact=1;
		for(Ki1=K.begin(), Kend1=K.end(); Ki1!=Kend1 && isNewContact; Ki1++)
			if(contactObject->alpha==(*Ki1)->alpha){
				isNewContact=0;
				(*Ki1)->v+=rt;
			}
		if(isNewContact){
			K.push_back(new LinkContactInfo(contactObject->alpha, rt));
		}
	}
	//increase Q size if needed
	nContacts=K.size();
	Q.resize(nContacts);
	//compute the Q's
	int i=0;
	for(Ki1=K.begin(), Kend1=K.end(); Ki1!=Kend1; Ki1++){
		Q[i].alpha=(*Ki1)->alpha;
		solveSystem(IStar, determinant, (*Ki1)->v, Q[i].v);
		i++;
	}
	//order the Q vector
	std::sort(Q.begin(), Q.end());
	/*
	//fill the contact matrix for the body: se face la apelarea din Simulator::fillContactMatrix
	for(contacts.rewindIndex();contacts.notAtEnd();contacts.advanceIndex()){
		fillContactMatrix(contacts.getCurrentElement());
	}
	*/
}


void ArticulatedAgentQuasistatic::computeBodyDerivativesWithoutContacts(real determinant){
	//compute the velocity of the body: requires inversion of a symmetric matrix	
	solveSystem(IStar, determinant, betaStar, beta);
	omega=beta.x;
	vLocal.setXY(beta.y, beta.z);
	v=vLocal;
	v.rotate(alpha);
}

void ArticulatedAgentQuasistatic::forwardAccelerations(ContactSolver* contactSolver){
	ArticulatedLink* link;
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd;
	ArticulatedLinkPVector::iterator linksI, linksEnd;
	Vector2 vLocalParent, vLocalParentRotated;
	real omegaParent, omegaLParent;
	real iq;
	real lParent;
	Vector2 vt1, vt2;
	ContactInfo* contactInfo;

    unsigned i;
	for(limbsI=limbs.begin(), limbsEnd=limbs.end(); limbsI!=limbsEnd; limbsI++){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; linksI++){
			link=*linksI;
			assert(link);
			//update the link Q number of elements
			link->Q.resize(nContacts);	
			link->nContacts=nContacts;
			if(linksI==limb->links.begin()) {
				vLocalParent=vLocal;
				vLocalParent.rotate(-limb->theta);
				omegaParent=omega;
				lParent=limb->l;				
				for (i = 0; i < nContacts; i++) {
					link->Q[i]=Q[i];
					link->Q[i].v.rotate(-limb->theta);
				}
			} else {
				vLocalParent=link->parentLink->vLocal;
				omegaParent=link->parentLink->omega;
				lParent=link->parentLink->l;
				for ( i=0; i < nContacts; i++) {
					link->Q[i]=link->parentLink->Q[i];
				}
			}
			omegaLParent=omegaParent*lParent;
			//rotate vLocalParent with the spatial rotation matrix; this includes the actual rotation, 
			//and the w l term
			vLocalParentRotated.setXY(	
						(omegaLParent+vLocalParent.y)*link->sinTheta+vLocalParent.x*link->cosTheta,
						(omegaLParent+vLocalParent.y)*link->cosTheta-vLocalParent.x*link->sinTheta);
			//update omega
			link->omega=(link->motorTorque+link->betaStar.x-
				(	link->IStar.getElement(1,2)*vLocalParentRotated.x+
					link->IStar.getElement(1,3)*vLocalParentRotated.y))/link->IStar.getElement(1,1);
			//update thetaD
			link->thetaD=link->omega-omegaParent;
			//update vLocal
			link->vLocal=vLocalParentRotated;
			//update v
			link->v=link->vLocal;
			link->v.rotate(link->alpha);
			
			//update Q
			LinkContactInfoVector::iterator Qi, Qend;
			for(Qi=link->Q.begin(), Qend=link->Q.end(); Qi!=Qend; Qi++){
				//rotate Q
				Vector3& vt=Qi->v;
				vt.setXYZ(	vt.x,
								(vt.x*lParent+vt.z)*link->sinTheta+vt.y*link->cosTheta,
								(vt.x*lParent+vt.z)*link->cosTheta-vt.y*link->sinTheta);
				iq=-(	link->IStar.getElement(1,1)*vt.x+
						link->IStar.getElement(1,2)*vt.y+
						link->IStar.getElement(1,3)*vt.z);
				for(LinkContactInfoPVector::iterator Ki=link->K.begin(), Kend=link->K.end();
						Ki!=Kend; Ki++)
					if((*Ki)->alpha==Qi->alpha)
						iq+=(*Ki)->v.x;
				//TODO: de optimizat ca sa nu trebuiasca ciclul K in ciclul Q, sa mearga in paralel
				//cum poate fi K ordonat? trebuie vazut atat cu contacte externe, cat si interne
				//are rost atunci ordonarea lui Q?
				iq/=link->IStar.getElement(1,1);
				//add iq to Q
				vt.x+=iq;
				//set the thetaFactor
				Qi->thetaFactor=iq;
				if(link->torqueContact!=NULL){
					if(link->torqueContact->alpha==Qi->alpha){
						//add component to Q
						vt.x+=link->torqueContact->sigma/link->IStar.getElement(1,1);						
						//add component to thetaFactor
						Qi->thetaFactor+=link->torqueContact->sigma/link->IStar.getElement(1,1);
						//TODO: de folosit real temporar pentru a nu face impartirea de 2 ori
						//add component to the contact matrix
						contactSolver->contactMatrix[link->torqueContact->alpha][link->torqueContact->alpha]+=
							1.0f/link->IStar.getElement(1,1);
						//set the contact vector component
						contactSolver->contactVector[link->torqueContact->alpha]=
							link->torqueContact->sigma*link->thetaD;
					}
					//fill the contact matrix for the torque contact
					contactSolver->contactMatrix[link->torqueContact->alpha][Qi->alpha]+=
						link->torqueContact->sigma*iq;
				}				
			}
			//fill the matrix for this link
			for(ContactInfoPVector::iterator ci=link->contacts.begin(), 
						cend=link->contacts.end(); ci!=cend; ci++){
				contactInfo=*ci;
				for(LinkContactInfoVector::iterator Qi=link->Q.begin(), 
										Qend=link->Q.end(); Qi!=Qend; Qi++){
					vt1.setXY(Qi->v.y,Qi->v.z);
					vt1.rotate(link->alpha);
					vt2=contactInfo->p;
					vt2.cross(Qi->v.x);
					vt1+=vt2;
					contactSolver->contactMatrix[contactInfo->alpha][Qi->alpha]+=
						vt1*contactInfo->n*contactInfo->sigma;	
				}
				vt1=contactInfo->p;
				vt1.cross(link->omega);
				vt1+=link->v;
				contactSolver->contactVector[contactInfo->alpha]+=vt1*contactInfo->n*contactInfo->sigma;
			}
		}
	}
}



void ArticulatedAgentQuasistatic::computeDerivatives(GlobalContactInfoVector* globalContacts){
	real f;
	Vector2 vt;
	LinkContactInfoVector::iterator Qi, Qend;

	computeBodyDerivatives(globalContacts);

	setSensors();
	
	ArticulatedLink* link;
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd;
	ArticulatedLinkPVector::iterator linksI, linksEnd;
	for(limbsI=limbs.begin(), limbsEnd=limbs.end(); limbsI!=limbsEnd; limbsI++){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; linksI++){
			link=*linksI;
			assert(link);
			for (Qi=link->Q.begin(), Qend=link->Q.end(); Qi!=Qend; Qi++){
				f=(*globalContacts)[Qi->alpha].force;
				/*
				link->omega+=link->Q[i].v.x*f;
				link->vLocal.x+=link->Q[i].v.y*f;
				link->vLocal.y+=link->Q[i].v.z*f;
				*/
				link->thetaD+=Qi->thetaFactor*f;
			}
			link->setSensors();
		}
	}
	//TEST: computes the total force that act on each link
	//computeForces(globalContacts);
}

void ArticulatedAgentQuasistatic::computeBodyDerivatives(GlobalContactInfoVector* globalContacts){
	real f;
	Vector2 vt;
	LinkContactInfoVector::iterator Qi, Qend;
	//update body velocity
	for (Qi=Q.begin(), Qend=Q.end(); Qi!=Qend; Qi++){
		f=(*globalContacts)[Qi->alpha].force;
		omega+=Qi->v.x*f;
		vt.setXY(Qi->v.y*f,Qi->v.z*f);
		vt.rotate(alpha);
		v+=vt;
	}
}

void ArticulatedAgentQuasistatic::computeForces(GlobalContactInfoVector* globalContacts){
	ArticulatedLink* link;
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd;
	ArticulatedLinkPVector::iterator linksI, linksEnd;
	for(limbsI=limbs.begin(), limbsEnd=limbs.end(); limbsI!=limbsEnd; limbsI++){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; linksI++){
			link=*linksI;
			assert(link);
			link->computeForceQuasistatic(globalContacts);
		}
	}
}

void ArticulatedAgentQuasistatic::computeTotalForces(GlobalContactInfoVector* globalContacts){
	
	ArticulatedLink* link;
	ArticulatedLink* childLink;
	ArticulatedLinkPVector::iterator cli, clend;
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd;
	ArticulatedLinkPVector::iterator linksI, linksEnd;	
	

	for(limbsI=limbs.begin(), limbsEnd=limbs.end(); limbsI!=limbsEnd; limbsI++){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; linksI++){
			link=*linksI;
			assert(link);
			link->computeForceQuasistatic(globalContacts);
			link->totalTorque=link->motorTorque;
			link->totalForce=link->force;
			for(cli=link->childLinks.begin(), clend=link->childLinks.end(); cli!=clend; cli++){
				childLink=*cli;
				assert(childLink);
				link->totalForce-=childLink->force;
				link->totalTorque-=childLink->motorTorque+link->l*(
					childLink->sinTheta*childLink->forceLocal.x+
					childLink->cosTheta*childLink->forceLocal.y);
			}
			/*
			Vector2 vt;
			vt.x=-link->s.y;	
			vt.y=link->s.x;
			vt*=link->omega;
			vt+=link->vLocal;
			vt*=link->m;
			vt.rotate(link->alpha);
			//TEST: tests that the forces and torques computed with the Featherstone algorithm verify 
			//the dynamic ecuations
			wxString string;		
			string.Printf("%10f\t%10f\t%10f\n",
				link->totalForce.x-vt.x,
				link->totalForce.y-vt.y,
				link->totalTorque-
				(link->I*link->omega+link->m*(-link->s.y*link->vLocal.x+link->s.x*link->vLocal.y))
				);
			if(simulator->textControl)
				simulator->textControl->AppendText(string);

			*/
		}
		/*

		if(simulator->textControl)
			simulator->textControl->AppendText("\n");

		*/
	}
}

void ArticulatedAgentQuasistatic::integrate(const Integrator &integrator) {
	ArticulatedLink* link;
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd=limbs.end();
	ArticulatedLinkPVector::iterator linksI, linksEnd;	

	for(limbsI=limbs.begin(); limbsI!=limbsEnd; ++limbsI){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; ++linksI){
			link=*linksI;
			assert(link);
			link->integrate(integrator); //integrates only theta, based on thetaD
		}
	}
	ComposedPhysicalObject::integrate(integrator);
	forwardKinematics();
	
	//now the bounding box is the box of the base
	for(limbsI=limbs.begin(); limbsI!=limbsEnd; ++limbsI){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; ++linksI){
			link=*linksI;
			boxMin.updateMin(link->boxMin);
			boxMax.updateMax(link->boxMax);
		}
	}	
}

void ArticulatedAgentQuasistatic::rollback(){
	ArticulatedLink* link;
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd=limbs.end();
	ArticulatedLinkPVector::iterator linksI, linksEnd;	

	
	for(limbsI=limbs.begin(); limbsI!=limbsEnd; ++limbsI){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; ++linksI){
			link=*linksI;
			assert(link);
			link->rollback();
		}
	}
	ComposedPhysicalObject::rollback();
	//now the bounding box is the box of the base
	for(limbsI=limbs.begin(); limbsI!=limbsEnd; ++limbsI){
		limb=*limbsI;
		for (linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; ++linksI){
			link=*linksI;
			boxMin.updateMin(link->boxMin);
			boxMax.updateMax(link->boxMax);
		}
	}
}







