/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef ARTICULATEDLINK_H
#define ARTICULATEDLINK_H

#include "ArticulatedComponent.h"


class Simulator;

/** Default elastic constant of the link articulation */
const real DEFAULT_K=(real)0.01; //0.01
/** Default damping factor of the link articulation */
const real DEFAULT_ETA=(real)0.15;


class ArticulatedLimb;

class ArticulatedLink;
typedef std::vector<ArticulatedLink*> ArticulatedLinkPVector;


/** A link that composes an articulated agent. A link has an articulation on which the agent can 
produce a torque. */

class ArticulatedLink : public ArticulatedComponent  {
public:
	ArticulatedLink(const std::string &label);
	virtual ~ArticulatedLink();
	
	/** Computes the torque and force that acts on the link (from IStar and betaStar), 
	for the quasistatical physics case. */
	void computeForceQuasistatic(GlobalContactInfoVector* globalContacts);

	/** Normalizes theta to the interval (-Pi,Pi) around theta0 */
	void normalizeTheta();

	/** Updates the values of sin, cos, sin2, cos2, sinCos as a function of theta*/
	void computeSinCos();

	/** Advances the time to the next timestep. A further call to forward kinematics is needed for updating
	the global coordinates. */
	void integrate(const Integrator &integrator);

	/** Rolls back the time to the previous timestep. */
	void rollback(){
		theta=thetaOld;
		ComposedPhysicalObject::rollback();
	}


	/** Determines whether the torque should be computed as an elastic torque given by theta and theta0
	(if true) or whether the torque is given at each timestep (if false)*/
	bool hasElasticTorque;

	/** Computes the motor torque, given theta and (target theta) theta0, as an elastic torque. */
	void computeMotorTorque();

	void setParentLink(ArticulatedLink* parentLink);

	/** Does nothing. The contact matrix filling for links is performed in 
	ArticulatedAgent::forwardAccelerations. */
	void fillContactMatrix(ContactSolver* contactSolver, ContactInfo* contactInfo) {
    }
	
	ContactInfo* torqueContact;
	void deleteContacts(){ 
		PhysicalObject::deleteContacts(); 
		if(torqueContact!=NULL){
			delete torqueContact;
			torqueContact=NULL;
		}
		force.setToZero();
	}

	void detectTorqueContact(GlobalContactInfoVector* contacts);

	void draw(GUI *gui);	

	/** The total torque that acts on the link, relative to its reference point. */
	real totalTorque;
	/** The force that is exercited on the link by its parent link, in laboratory reference system. */
	Vector2 force;
	/** The force that is exercited on the link by its parent link, in link reference system. */
	Vector2 forceLocal;
	/** The total force that is exercited on the link, in laboratory reference system. */
	Vector2 totalForce;

	/** The rotation angle of the link relative to the coordinate system of the predecessor link.
	It is the generalized coordinate of the link. */
	real theta;
	real thetaOld;

	/** Functions of theta */
	real sinTheta, cosTheta, sin2, cos2, sinCos;

	/** The Coriolis force. */
	Vector3 coriolis;

	/** Temporal derivate of theta */
	real thetaD;

	real thetaDD;

	/** The equilibrium position of the link, commanded by the motor neurons */
	real theta0;
	/** Minimum and maximum values of theta0 */
	real theta0Min, theta0Max;
	/** The maximum permitted displacement relative to the equilibrium position of the link */
	real deltaThetaMax;
	/** The motor torque applied at the link joint */
	real motorTorque;

	/** Elastic constant of the link articulation */
	real k;
	/** Damping factor of the link articulation */
	real eta;

	/** Length of the link along its x axis */
	real l;	

	/** The parent link. */
	ArticulatedLink* parentLink;
	
	/** A list of child links. The list does NOT own the links. It should NOT
	delete the child links on destruction. */
	ArticulatedLinkPVector childLinks;
};




#endif //ARTICULATEDLINK_H
